<?php

/**
 * Class ModelExtensionEgoiEgoi
 * @author E-goi
 */
class ModelExtensionEgoiEgoi extends Model
{
    /**
     * @var string
     */
    private $code = "egoi";

    /**
     * @var string
     */
    private $name = "Smart Marketing";

    /**
     * @var string
     */
    private $author = "E-goi";

    /**
     * @var string
     */
    private $author_link = "https://www.e-goi.com";

	/**
	 * @var int
	 */
	private $version = 133;

	/**
	 * @param $version
	 */
    public function install($version)
    {
        $this->update($version);
    	$this->load->model('setting/event');
		$this->model_setting_event->deleteEventByCode('smart_marketing_notifications');
		$this->model_setting_event->deleteEventByCode('smart_marketing_reminders');
    	$this->model_setting_event->addEvent('smart_marketing_notifications', 'catalog/model/checkout/order/addOrderHistory/after', 'extension/module/smart_marketing_notifications/processNotifications');
		$this->model_setting_event->addEvent('smart_marketing_reminders', 'catalog/view/common/header/before', 'extension/module/smart_marketing_reminders/processReminders');
    }

	/**
	 * uninstall
	 */
	public function uninstall(){
		$this->load->model('setting/event');
		$this->model_setting_event->deleteEventByCode('smart_marketing_notifications');
		$this->model_setting_event->deleteEventByCode('smart_marketing_reminders');
	}

    /**
     * @param $version
     */
    public function update($version)
    {
        if($version >= 100){
            $this->update100();
        }
    }

    /**
     * @param $version
     * @param $xml
     * @param bool $update
     */
    private function setModification($version, $xml, $update=false)
    {
        if($update){
            $this->db->query("UPDATE `" . DB_PREFIX . "modification` SET xml = '" . $this->db->escape($xml). "' WHERE code = 'egoi' AND version = '" . $this->db->escape($version) . "'");
            return;
        }

        $this->db->query("INSERT INTO `" . DB_PREFIX . "modification`(name, code, author, version, link, xml, status, date_added) VALUES ('$this->name', '$this->code','$this->author', '" . $this->db->escape($version) . "', '$this->author_link', '" . $this->db->escape($xml) . "', '1', NOW())");
    }

    /**
     *
     */
    private function update100()
    {

		$this->db->query("DELETE FROM `" . DB_PREFIX . "setting` WHERE `KEY` = 'smart_marketing_sms_notifications_install_date'");

		$this->db->query("INSERT INTO " . DB_PREFIX . "setting SET store_id = '0', `code` = 'smart_marketing_sms_notifications', `key` = 'smart_marketing_sms_notifications_install_date', `value` = '" . date('Y-m-d H:i:s') . "'");

        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "egoi_forms` (
            `smart_marketing_form_id` int(11) NOT NULL,
            `smart_marketing_form_type` varchar(10) NOT NULL,
            `smart_marketing_form_title` varchar(100) NOT NULL,
            `smart_marketing_form_html_status` tinyint(4) NOT NULL,
            `smart_marketing_form_html_show_title` tinyint(4) NOT NULL,
            `smart_marketing_form_html_html` text NOT NULL,
            `smart_marketing_form_iframe_subscribe_list` int(11) NOT NULL,
            `smart_marketing_form_iframe_form` int(11) NOT NULL,
            `smart_marketing_form_styles_border_size` varchar(10) NOT NULL,
            `smart_marketing_form_styles_border_color` varchar(10) NOT NULL,
            `smart_marketing_form_styles_box_width` varchar(10) NOT NULL,
            `smart_marketing_form_styles_box_height` varchar(10) NOT NULL,
            PRIMARY KEY (`smart_marketing_form_id`)
        ) DEFAULT CHARSET=utf8;");

		$this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "egoi_sms_order_reminders` (
            `smart_marketing_reminder_id` int(11) NOT NULL AUTO_INCREMENT,
            `smart_marketing_reminder_date` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
            `smart_marketing_reminder_order_id` int(11) NOT NULL,
            `smart_marketing_reminder_status_id` int(11) NOT NULL,
            PRIMARY KEY (`smart_marketing_reminder_id`),
            UNIQUE KEY `unique_constraint` (`smart_marketing_reminder_order_id`,`smart_marketing_reminder_status_id`)
        ) DEFAULT CHARSET=utf8;");

		$has_egoi_trackingCode = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "order_history` WHERE Field='tracking_number'")->num_rows > 0;
		if (!$has_egoi_trackingCode) {
			$this->db->query("ALTER TABLE " . DB_PREFIX . "order_history ADD COLUMN tracking_number text NOT NULL");
		}

		$has_egoi_courier = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "order_history` WHERE Field='shipping_courier_id'")->num_rows > 0;
		if (!$has_egoi_courier) {
			$this->db->query("ALTER TABLE " . DB_PREFIX . "order_history ADD COLUMN shipping_courier_id int(11) NOT NULL");
		}

		$has_egoi_shipping_courier_uri = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "shipping_courier` WHERE Field='shipping_courier_tracking_uri'")->num_rows > 0;
		if (!$has_egoi_shipping_courier_uri) {
			$this->db->query("ALTER TABLE " . DB_PREFIX . "shipping_courier ADD COLUMN shipping_courier_tracking_uri text NOT NULL");
		}

    }

    /**
     * @param $status
     * @param $code
     */
    public function activateTrackAndEngage($status, $code)
    {
        $this->setPosition($status, $code, 'content_bottom');
    }

    /**
     * @param $status
     * @param $code
     * @param $position
     * @param $priority
     * @param string $limit_to
     */
    public function setPosition($status, $code, $position, $priority = '99', $limit_to = "")
    {
        if($status) {
            $this->load->model('design/layout');

            $res = $this->db->query("SELECT * FROM " . DB_PREFIX . "layout_module where code='$code'");
            if (!empty($res) && (isset($res->num_rows) && ($res->num_rows))) {
                $this->db->query("
                      UPDATE " . DB_PREFIX . "layout_module SET
                        position = '$position', 
                        sort_order = $priority WHERE code = '$code'
                    ");

            }else{
                foreach ($this->model_design_layout->getLayouts() as $layout) {
                    if(empty($limit_to) || strtolower($limit_to) == strtolower($layout["name"])) {
                        if(!$this->checkIfAlreadyHasLayout($layout['layout_id'], $position, $code)) {
                            $this->db->query("
                                  INSERT INTO " . DB_PREFIX . "layout_module SET
                                    layout_id = '" .$layout['layout_id']."',
                                    code = '$code',
                                    position = '$position', 
                                    sort_order = $priority 
                                ");
                        }
                    }
                }
            }

        }else{
            $this->db->query("DELETE FROM " . DB_PREFIX . "layout_module WHERE code = '$code'");
        }
    }

    /**
     * @param $table
     * @param $fields_values
     */
    public function setEgoiFormSetting($table, $fields_values)
    {
        $update = isset($fields_values["smart_marketing_form_id"]);

        $insert_fields = "";
        $insert_values = "";
        $update_fields_values = "";

        $i = 0;
        foreach ($fields_values as $field => $value){

            if($field != "smart_marketing_form_id" || $field != "smart_marketing_form_layout_position"){
                if($i++ > 0){
                    $insert_fields .= ", ";
                    $insert_values .= ", ";
                    $update_fields_values .= ", ";
                }

                // for inserts
                $insert_fields .= "$field";
                $insert_values .= "'". $this->db->escape($value) . "'";

                // for update
                $update_fields_values .= "$field = '". $this->db->escape($value) . "'";
            }
        }

        if($update) {
            $this->db->query("UPDATE " . DB_PREFIX . "$table SET $update_fields_values WHERE smart_marketing_form_id = " . $fields_values["smart_marketing_form_id"]);
            return;
        }

        $this->db->query("INSERT INTO " . DB_PREFIX . "$table ($insert_fields) VALUES ($insert_values)");
    }

    /**
     * @param $code
     * @param $data
     * @return mixed
     */
    public function addModule($code, $data)
    {
        $this->db->query("INSERT INTO `" . DB_PREFIX . "module` SET `name` = '" . $this->db->escape($data['name']) . "', `code` = '" . $this->db->escape($code) . "', `setting` = '" . $this->db->escape(json_encode($data)) . "'");

        return $this->db->getLastId();
    }

    /**
     * @param $layout_id
     * @param $position
     * @param $code
     * @return bool
     */
    private function checkIfAlreadyHasLayout($layout_id, $position, $code)
    {
        $query = $this->db->query("SELECT COUNT(*) AS total FROM " . DB_PREFIX . "layout_module WHERE code = '$code' AND layout_id = '" . (int)$layout_id . "' AND position = '$position'");

        return ((isset($query->row['total']) && $query->row['total']>0)?true:false);
    }
}