<?php
/**
 * @date    2019-05-15 10:42:11
 * @Copyright (c) E-goi <info@e-goi.com>
 * @Author  Renato Gomes <rgomes@e-goi.com>
 * @License Proprietary
 * @Version 1.0
 *
 * All rights reserved. 2019
 *
 */


include_once DIR_SYSTEM . "/library/egoi/egoi_api.php" ;

/**
 * Class ControllerExtensionModuleSmartMarketingNotifications
 * @author E-goi
 */
class ControllerExtensionModuleSmartMarketingNotifications extends Controller
{
	/**
	 * EGOI TEMPLATE DELIMITER
	 */
	const EGOI_TEMPLATE_DELIMITER = '%';

	/**
	 * @var array
	 */
	private $egoiTemplating = array(
		array(
			'label' => 'order_id',
			'eg'  => self::EGOI_TEMPLATE_DELIMITER . 'order_id' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'order_status_string',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'order_status' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'total',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'total' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'currency_code',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'currency' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'ref',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'ref' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'ent',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'ent' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'store_name',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'shop_name' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'firstname',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'billing_name' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'tracking_name',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'tracking_name' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'tracking_code',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'tracking_code' . self::EGOI_TEMPLATE_DELIMITER
		),
		array(
			'label' => 'tracking_url',
			'code'  => self::EGOI_TEMPLATE_DELIMITER . 'tracking_url' . self::EGOI_TEMPLATE_DELIMITER
		)
	);

	/**
	 * @param $route
	 * @param $data
	 * @param $output
	 *
	 * @return bool
	 * @throws SoapFault
	 */
	public function processNotifications(&$route, &$data, &$output)
	{

		if(empty($data[0])){
			return false;
		}
		if(empty($data[1])){
			return false;
		}

		$this->load->model('setting/setting');
		$this->load->model('checkout/order');
		$this->load->model('account/order');
		$this->load->model('localisation/order_status');

		$order = $this->model_checkout_order->getOrder($data[0]);

		$apiEgoi = new EgoiAPI($this->config->get('smart_marketing_api'));

		if(empty($order['language_code'])){
			$order['language_code'] = 'en-gb';
		}

		$order_status  = $this->db->query("SELECT name FROM " . DB_PREFIX . "order_status WHERE language_id = " . (int)$order['language_id'] . " AND order_status_id=". (int)$data[1] ." ORDER BY name LIMIT 1");

		$order['order_status_string'] = $order['order_status'];

		if(!empty($order_status)){
			$order_status = $order_status->rows;
			$order['order_status_string'] = $order_status[0]['name'];
		}

		$hasTrackingEnabled = $this->checkTrackingEnabled();

		$order['tracking_name'] = '';
		$order['tracking_code'] = '';
		$order['tracking_url'] = '';

		if(!empty($order['order_id']) && !empty($order['order_status_id']) && $hasTrackingEnabled){
			$order_history  = $this->db->query("SELECT shipping_courier_id, tracking_number FROM " . DB_PREFIX . "order_history WHERE order_id = " . (int)$order['order_id'] . " AND order_status_id=". (int)$order['order_status_id'] ." order by order_history_id desc LIMIT 1");

			if(!empty($order_history)){
				$order_history = $order_history->rows;
				$order['tracking_code'] = $order_history[0]['tracking_number'];

				if(!empty($order_history[0]['shipping_courier_id'])){
					$courier = $this->db->query("SELECT shipping_courier_id, shipping_courier_name, shipping_courier_tracking_uri FROM `" . DB_PREFIX . "shipping_courier` WHERE shipping_courier_id=".(int)$order_history[0]['shipping_courier_id']);
					if(!empty($courier)){
						$courier = $courier->rows;
						$order['tracking_name'] = $courier[0]['shipping_courier_name'];
						if(!empty($order['shipping_courier_tracking_uri'])){
							$order['tracking_url'] = $courier[0]['shipping_courier_tracking_uri'];
						} else {
							$order['tracking_url'] = 'https://www.17track.net/?nums='; //fail safe
						}
					}
				}
			}
		}

		if(!empty($order['tracking_code']) && empty($order['tracking_url'])){
			$order['tracking_url'] = 'https://www.17track.net/?nums='; //fail safe
		}

		$sender = $this->config->get('smart_marketing_sms_notifications_sms_sender');
		$sendTo = $apiEgoi->validateCellphone($order['telephone'], $order);

		if(empty($sender)){
			return false;
		}

		if(!empty($sendTo)){
			if(!empty($this->config->get('smart_marketing_sms_notifications_'.$data[1].'_customer_enable'))){

				if(!empty($this->config->get('smart_marketing_sms_notifications_'.$data[1].'_customer_message_'.$order['language_code']))){

					$messageCustomer = $this->config->get('smart_marketing_sms_notifications_'.$data[1].'_customer_message_'.$order['language_code']);

					foreach ($this->egoiTemplating as $eg) {
						if(empty($eg['code']) || empty($order[$eg['label']])) continue;
						$messageCustomer = str_replace($eg['code'], $order[$eg['label']], $messageCustomer);
					}

					$apiEgoi->sendSms(
						$sendTo,
						$sender,
						$messageCustomer);

				}

			}
		}

		$sendTo = $this->config->get('smart_marketing_sms_notifications_admin_sender');

		if (!empty($sendTo)) {
			if (!empty($this->config->get('smart_marketing_sms_notifications_' . $data[1] . '_admin_enable'))) {

				if (!empty($this->config->get('smart_marketing_sms_notifications_' . $data[1] . '_admin_message_' . $order['language_code']))) {

					$messageAdmin = $this->config->get('smart_marketing_sms_notifications_' . $data[1] . '_admin_message_' . $order['language_code']);

					foreach ($this->egoiTemplating as $eg) {
						if (empty($eg['code']) || empty($order[$eg['label']])) continue;
						$messageAdmin = str_replace($eg['code'], $order[$eg['label']], $messageAdmin);
					}

					$apiEgoi->sendSms(
						$sendTo,
						$sender,
						$messageAdmin);

				}
			}
		}

	}

	/**
	 * @param $route
	 * @param $data
	 * @param $output
	 *
	 * @return bool
	 * @throws SoapFault
	 */
	public function processReminders(&$route, &$data, &$output)
	{

		if (date('G') >= 10 && date('G') <= 22) {
			$this->load->model('setting/setting');
			$this->load->model('checkout/order');

			$apiEgoi = new EgoiAPI($this->config->get('smart_marketing_api'));
			$sender = $this->config->get('smart_marketing_sms_notifications_sms_sender');
			$interval = $this->config->get('smart_marketing_sms_notifications_reminder_interval');

			$installDate = $this->config->get('smart_marketing_sms_notifications_install_date');

			if(empty($sender)){
				return false;
			}

			if(empty($interval)){
				$interval = '48';
			}

			if(empty($installDate)){
				$installDate = date('Y-m-d H:i:s');
			}

//			$queryDate = date('Y-m-d H:i:s', strtotime('-'.$interval.' hour'));

			$order_statuses  = $this->db->query("SELECT DISTINCT(order_status_id) as order_status_id FROM " . DB_PREFIX . "order_status");
			$numProcessed = 0;

			if (!empty($order_statuses)) {
				$order_statuses = $order_statuses->rows;

				if (!empty($order_statuses)) {
					foreach ($order_statuses as $order_status) {
						if($numProcessed > 20){
							break;
						}


						//2019-05-13 10:55:00


						if (!empty($this->config->get('smart_marketing_sms_notifications_' . $order_status["order_status_id"] . '_reminder_enable'))) {
//							$allOrderForStatus =  $this->db->query("SELECT o.order_id FROM " . DB_PREFIX . "order o WHERE o.order_status_id=".$order_status["order_status_id"]." AND o.date_added < '" . $queryDate . "' AND o.order_id NOT IN (SELECT esor.smart_marketing_reminder_order_id FROM " . DB_PREFIX . "egoi_sms_order_reminders esor WHERE esor.smart_marketing_reminder_order_id=o.order_id AND esor.smart_marketing_reminder_status_id=".$order_status["order_status_id"].")");
							$allOrderForStatus =  $this->db->query("SELECT o.order_id FROM " . DB_PREFIX . "order o WHERE o.order_status_id=".$order_status["order_status_id"]." AND  o.order_id NOT IN (SELECT esor.smart_marketing_reminder_order_id FROM " . DB_PREFIX . "egoi_sms_order_reminders esor WHERE esor.smart_marketing_reminder_order_id=o.order_id AND esor.smart_marketing_reminder_status_id=".$order_status["order_status_id"].") AND o.date_added>='".$installDate."'");

							if (!empty($allOrderForStatus)) {
								$allOrderForStatus = $allOrderForStatus->rows;
								if (!empty($allOrderForStatus)) {

									foreach ($allOrderForStatus as $allOrderForStatus) {

										if($numProcessed > 20){
											break;
										}

										$numProcessed++;

										$order = $this->model_checkout_order->getOrder($allOrderForStatus['order_id']);

										if(empty($order['language_code'])){
											$order['language_code'] = 'en-gb';
										}

										$order_status_string  = $this->db->query("SELECT name FROM " . DB_PREFIX . "order_status WHERE language_id = " . (int)$order['language_id'] . " AND order_status_id=". (int)$order["order_status_id"] ." ORDER BY name LIMIT 1");

										$order['order_status_string'] = $order['order_status'];

										if(!empty($order_status_string)){
											$order_status_string = $order_status_string->rows;
											$order['order_status_string'] = $order_status_string[0]['name'];
										}

										$hasTrackingEnabled = $this->checkTrackingEnabled();

										$order['tracking_name'] = '';
										$order['tracking_code'] = '';
										$order['tracking_url'] = '';

										if(!empty($order['order_id']) && !empty($order['order_status_id']) && $hasTrackingEnabled){
											$order_history  = $this->db->query("SELECT shipping_courier_id, tracking_number FROM " . DB_PREFIX . "order_history WHERE order_id = " . (int)$order['order_id'] . " AND order_status_id=". (int)$order['order_status_id'] ." order by order_history_id desc LIMIT 1");

											if(!empty($order_history)){
												$order_history = $order_history->rows;
												$order['tracking_code'] = $order_history[0]['tracking_number'];

												if(!empty($order_history[0]['shipping_courier_id'])){
													$courier = $this->db->query("SELECT shipping_courier_id, shipping_courier_name, shipping_courier_tracking_uri FROM `" . DB_PREFIX . "shipping_courier` WHERE shipping_courier_id=".(int)$order_history[0]['shipping_courier_id']);
													if(!empty($courier)){
														$courier = $courier->rows;
														$order['tracking_name'] = $courier[0]['shipping_courier_name'];
														if(!empty($order['shipping_courier_tracking_uri'])){
															$order['tracking_url'] = $courier[0]['shipping_courier_tracking_uri'];
														} else {
															$order['tracking_url'] = 'https://www.17track.net/?nums='; //fail safe
														}
													}
												}
											}
										}

										if(!empty($order['tracking_code']) && empty($order['tracking_url'])){
											$order['tracking_url'] = 'https://www.17track.net/?nums='; //fail safe
										}

										$sendTo = $apiEgoi->validateCellphone($order['telephone'], $order);

										if(!empty($this->config->get('smart_marketing_sms_notifications_'.$order["order_status_id"] .'_customer_message_reminder_'.$order['language_code']))){

											$messageCustomer = $this->config->get('smart_marketing_sms_notifications_'.$order["order_status_id"] .'_customer_message_reminder_'.$order['language_code']);


											if($order['payment_code'] == 'eupagomultibanco' && !empty($this->config->get('payment_eupagomultibanco_authkey'))){

												$eupagoData =  $this->db->query("SELECT * FROM " . DB_PREFIX . "order_history WHERE order_id=".$order["order_id"]." AND order_status_id = " . $order_status["order_status_id"]);

												if(!empty($eupagoData)){
													$eupagoData = $eupagoData->rows;

													if(!empty($eupagoData[0]['comment'])){

														$eupagoData = strip_tags($eupagoData[0]['comment']);

														$euPagoParse = explode(':', $eupagoData);

														foreach ($euPagoParse as $k =>$v) {
															$euPagoParse[$k] = (int) str_replace(' ', '', $v);
														}

														$order['ent'] = !empty($euPagoParse[1]) ? $euPagoParse[1] : '';
														$order['ref'] = !empty($euPagoParse[2]) ? $euPagoParse[2] : '';

													}
												}

											}

											$order['total'] = $this->currency->format($order['total'], $order['currency_code'], $order['currency_value'], false);

											foreach ($this->egoiTemplating as $eg) {
												if(empty($eg['code']) || empty($order[$eg['label']])) continue;
												$messageCustomer = str_replace($eg['code'], $order[$eg['label']], $messageCustomer);
											}

											//chegou aqui
											//insere na tabela de reminders
											try {
												$sql = "INSERT INTO " . DB_PREFIX . "egoi_sms_order_reminders (smart_marketing_reminder_date, smart_marketing_reminder_order_id, smart_marketing_reminder_status_id) VALUES ";
												$sql.= " ('" . date('Y-m-d H:i:s') . "', " . (int)$order['order_id'] . ", " . (int)$order['order_status_id'] . ") ";
												$this->db->query($sql);
											} catch (Exception $e) {
												continue;
											}

											$apiEgoi->sendSms(
												$sendTo,
												$sender,
												$messageCustomer);

										}
									}
								}
							}
						}
					}
				}
			}
		}
	}


	protected function checkTrackingEnabled() {
		$has_egoi_shipping_courier_uri = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "shipping_courier` WHERE Field='shipping_courier_tracking_uri'")->num_rows > 0;
		$has_egoi_trackingCode = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "order_history` WHERE Field='tracking_number'")->num_rows > 0;
		$has_egoi_courier = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "order_history` WHERE Field='shipping_courier_id'")->num_rows > 0;

		return !empty($has_egoi_shipping_courier_uri && $has_egoi_trackingCode && $has_egoi_courier);

	}



}