<?php

/**
 * Class EgoiAPI
 * @author E-goi
 */
class EgoiAPI
{
    /**
     * @var string PLUGIN_KEY
     */
    const PLUGIN_KEY = 'fc9b0a078d98eb4f747f888e681b81c7';

    /**
     * @var string API_URL
     */
    const API_URL = 'https://api.e-goi.com/v2/soap.php?wsdl';

	/**
	 * @var string API_V3_URL
	 */
	const API_V3_URL = 'https://api.egoiapp.com';

	/**
	 * @var string API_TRANSACTIONAL_URL
	 */
	const API_TRANSACTIONAL_URL = 'https://www51.e-goi.com/api/public';

	/**
	 * @var string PLUGIN_CURRENT_VERSION_URL
	 */
	const PLUGIN_CURRENT_VERSION_URL = 'https://goidini.e-goi.com/plugin-versions/current.json';

	/***
	 * @var string MAX_MESSAGES
	 */
	const MAX_MESSAGES = 10;

    /**
     * @var string
     */
    public $apiKey;

    /**
     * @var SoapClient
     */
    private $_client;

	/**
	 * @var int
	 */
	private $version = 133;

    /**
     * @var array
     */
    private $hiddenChars = ['-', '(', ')', ' ', '+'];

    /**
     * @var array
     */
    private $prefix = [
        'en' => '44-',
        'es' => '34-',
        'br' => '55-',
        'pt' => '351-'
    ];

	/**
	 * EgoiAPI constructor.
	 *
	 * @param $api_key
	 *
	 * @throws SoapFault
	 */
    public function __construct($api_key)
    {
        $this->apiKey = $api_key;
        $this->_client = new SoapClient(self::API_URL);
    }

    /**
     * @return null
     */
    public function getApiKey()
    {
        return $this->apiKey ?: null;
    }

    /**
     * @return array
     */
    private function getSoapParams()
    {
        return array(
            'plugin_key' => self::PLUGIN_KEY,
            'apikey' => $this->getApiKey()
        );
    }

    /**
     * @return mixed
     */
    public function checkLogin()
    {
        return $this->_client->checklogin($this->getSoapParams());
    }

    /**
     * @return mixed
     */
    public function getUserData()
    {
        return $this->_client->getUserData($this->getSoapParams());
    }

    /**
     * @return mixed
     */
    public function getClientData() 
    {
		$this->apiv3Call('RAWPOST', '/ping');
		$this->transactionalCall('RAWPOST', '/client');
        return $this->_client->getClientData($this->getSoapParams());
    }

	/**
	 * @return array
	 */
    public function getAccountData() {
    	return $this->apiv3Call('GET', '/my-account');
	}

	/**
	 * @return mixed
	 */
	public function getEcommerceCatalogs()
	{
		return $this->apiv3Call('GET', '/catalogs');
	}

	/**
	 * @param $catalogId
	 *
	 * @param $data
	 *
	 * @return array|bool|mixed
	 */
	public function importCatalog($catalogId, $data)
	{
		return $this->apiv3Call('RAWPOST', '/catalogs/'.$catalogId.'/products/actions/import', $data);
	}


	/**
	 * @return mixed
	 */
	public function postEcommerceCatalogs($data)
	{
		return $this->apiv3Call('RAWPOST', '/catalogs', $data);
	}

	/**
	 * @return mixed
	 */
	public function deleteEcommerceCatalogs($catalogId)
	{
		return $this->apiv3Call('DELETE', '/catalogs/'.$catalogId);
	}

	/**
	 * @return mixed
	 */
	public function syncEcommerceProducts($catalogId, $data)
	{
		return $this->apiv3Call('DELETE', '/catalogs/'.$catalogId.'/products/action/import', $data);
	}

	/**
	 * @return mixed
	 */
	public function getCountrysAndCurrencys()
	{
		return $this->apiv3Call('GET', '/utilities/countries');

	}



    /**
     * Create E-goi List
     *
     * @param $name
     * @param $lang
     * @return mixed
     */
    public function createList($name, $lang)
    {
        $params = array(
            'nome' => $name,
            'idioma_lista' => $lang
        );

        $result = $this->_client->createList(array_merge($this->getSoapParams(), $params));
        if(isset($result['ERROR']) && ($result['ERROR'])) {
            return $result['ERROR'];
        } else {
            return $result['LIST_ID'];
        }
    }

    /**
     * @param $list
     * @param bool|string $return_field
     * @return mixed
     */
    public function getList($list, $return_field = false)
    {
        $res = $this->_client->getLists(
            array_merge(
                $this->getSoapParams(), ['listID' => $list]
            )
        );
        if (!empty($res)) {
            foreach ($res as $val) {
                if ($return_field) {
                    return $val[$return_field];
                }
                return $val;
            }
        }
    }

    /**
     * Get all lists
     *
     * @return mixed
     */
    public function getLists()
    {
        return $this->_client->getLists($this->getSoapParams());
    }

    /**
     * Get All Active Subscribers Count on Specific List
     *
     * @param $listId
     * @return bool|int
     */
    public function getAllSubscribers($listId)
    {
        if (!empty($lists = $this->getLists())) {
            foreach ($lists as $list) {
                if ($list['listnum'] == $listId) {
                    return $list['subs_activos'];
                }
            }
        }
        return false;
    }

    /**
     * Get Forms from List
     *
     * @param string|int $listId
     * @param bool $option
     * @return array
     */
    public function getForms($listId, $option = false)
    {
        $forms = $this->_client->getForms(
            array_merge(
                $this->getSoapParams(),
                array('listID' => $listId)
            )
        );
        if($option){
            $array = array();
            if (!empty($forms) && !isset($forms['ERROR'])) {
                foreach($forms as $key => $value){
                    $array[$key]['id'] = $value['id'];
                    $array[$key]['title'] = $value['title'] . ' - ' . $value['url'];
                }
            }
            return $array;

        }
        return $forms;
    }

    /**
     * Get subscriber from List
     *
     * @param $listId
     * @param $email
     * @return mixed
     */
    public function getSubscriber($listId, $email)
    {
        $subs = $this->_client->subscriberData(
            array_merge(
                $this->getSoapParams(),
                array('listID' => $listId, 'subscriber' => $email)
            )
        );
        return $subs;
    }

    /**
     * @param $id
     * @param $fname
     * @param $lname
     * @param $birthday
     * @param $email
     * @param $list
     * @param string $formID
     * @param array $extraFields
     * @return bool
     */
    public function addSubscriber($id, $fname, $lname, $birthday, $email, $list, $formID = '', $extraFields = array())
    {
        $params = array(
            'listID' => $list,
            'subscriber' => $id,
            'status' => '1',
            'from' => '',
            'email' => $email,
            'validate_email' => '0',
            'cellphone' => '',
            'telephone' => '',
            'fax' => '',
            'formID' => $formID ?: '',
            'first_name' => $fname,
            'last_name' => $lname,
            'birth_date' => ($birthday) ? date('Y-m-d', strtotime($birthday)):'',
        );

        // check for extra fields
        if($extraFields AND count($extraFields) > 0)
            $params = array_merge($params, $extraFields);

        $result = $this->_client->addSubscriber(
            array_merge($this->getSoapParams(), $params)
        );

        if (isset($result['UID']) && ($result['UID'])) {
            return $result['UID'];
        }
        return false;
    }

    /**
     * @param array $params
     * @param bool|int $list_id
     * @return array
     */
    public function prepareSubscriber($params = array(), $list_id, $dbObject)
    {
        $subscriber = array();
        if(isset($params['email']) && !empty($params['email'])) {
            $subscriber["email"] = $params['email'];
        }

        if(isset($params['firstname']) && !empty($params['firstname'])) {
            $subscriber["first_name"] = $params['firstname'];
        }

        if(isset($params['lastname']) && !empty( $params['lastname'])) {
            $subscriber["last_name"] = $params['lastname'];
        }

        if(isset($params['birthday']) && !empty($params['birthday'])) {
            $subscriber["birthday"] = $params['birthday'];
        }

        if(isset($params['telephone']) && !empty($params['telephone'])) {

            // get current list lang
            $lang = $this->getList($list_id, 'idioma');
            if ($lang) {
                $number = $this->prefix[$lang] . str_replace($this->hiddenChars, '', $params['telephone']);

                // check if has a custom field as cellphone
                if(isset($params['custom_field']) && !empty($params['custom_field'])) {

                    foreach (json_decode($params['custom_field']) as $id => $val) {

                        $custom_field = $dbObject->query("SELECT name FROM oc_custom_field_description WHERE custom_field_id='$id'");
                        if (is_object($custom_field) && $custom_field->num_rows) {
                            if ($custom_field->row['name'] == 'Celular' || $custom_field->row['name'] == 'Cellphone') {
                                $subscriber["cellphone"] = $this->prefix[$lang] . str_replace($this->hiddenChars, '', $val);
                            }
                        }
                    }
                    
                    $subscriber["telephone"] = $number;

                }else{
                    $subscriber["cellphone"] = $number;
                }
            }
        }

        $subscriber['validate_phone'] = 0;

        return $subscriber;
    }

    /**
     * @param $id_list
     * @param array $subscribers
     * @param bool|array|int $tags
     * @return bool
     */
    public function addSubscriberBulk($id_list, $subscribers = array(), $tags = false)
    {
        $params = array(
            'listID' => $id_list,
            'compareField' => 'email',
            'operation' => 2,
            'subscribers' => $subscribers,
            'tags' => is_array($tags) ? $tags : array($tags)
        );

        $this->_client->addSubscriberBulk(array_merge($this->getSoapParams(), $params));
        return true;
    }

    /**
     * @param $list
     * @param $email
     * @param string $fname
     * @param string $lname
     * @param array $fields
     * @return mixed
     */
    public function editSubscriber($list, $email, $fname = '', $lname = '', $fields = array())
    {
        $params = array(
            'listID' => $list,
            'subscriber' => $email,
            'email' => $email,
            'first_name' => $fname,
            'last_name' => $lname
        );

        if(!empty($fields)) {
            foreach ($fields as $key => $value) {
                if($key == 'birth_date'){
                    $value = date('Y-m-d', strtotime($value));
                }
                $params[$key] = $value;
            }
        }

        return $this->_client->editSubscriber(array_merge($this->getSoapParams(), $params));
    }

    /**
     * @param $list
     * @param $email
     * @return mixed
     */
    public function removeSubscriber($list, $email)
    {
        $params = array(
            'listID' => $list,
            'subscriber' => $email
        );

        return $this->_client->removeSubscriber(array_merge($this->getSoapParams(), $params));
    }

    /**
     * @param $id_list
     * @param $name
     * @param $type
     * @return bool
     */
    public function createExtraField($id_list, $name, $type)
    {
        $params = array(
            'listID' => $id_list,
            'name' => $name,
            'type' => $type
        );

        $result = $this->_client->addExtraField(
            array_merge($this->getSoapParams(), $params)
        );

        if (isset($result['NEW_ID']) && ($result['NEW_ID'])) {
            return $result['NEW_ID'];
        }
        return false;
    }

    /**
     * @return mixed
     */
    public function getMappedFields()
    {
        $sql = "SELECT * FROM "._DB_PREFIX_."egoi_map_fields order by id DESC";
        $rq = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        return $rq;
    }

    /**
     * @param bool $name
     * @param bool $field
     * @return mixed
     */
    public function getFieldMap($name = false, $field = false)
    {
        if($field) {
            $sql = "SELECT * FROM "._DB_PREFIX_."egoi_map_fields WHERE ps='".pSQL($field)."'";
        }else {
            $sql = "SELECT * FROM "._DB_PREFIX_."egoi_map_fields WHERE egoi='".pSQL($name)."'";
        }
        $rq = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        return $rq[0]['egoi'];
    }

    /**
     * @param $listID
     * @return mixed
     */
    public function getExtraFields($listID)
    {
        $params = array(
            'listID' => $listID
        );

        $result_client = $this->_client->getExtraFields(array_merge($this->getSoapParams(), $params));
        return $result_client['extra_fields'];
    }

    /**
     * @param $name
     * @return mixed
     */
    public function addTag($name)
    {
        $params = array(
            'name' => $name,
        );
        return $this->_client->addTag(
            array_merge($this->getSoapParams(), $params)
        );
    }

    /**
     * @return mixed
     */
    public function getTags()
    {
        $result = $this->_client->getTags(array_merge($this->getSoapParams()));
        return $result['TAG_LIST'];
    }

    /**********************************************/
    /********** APIV3 AND TRANSACTIONAL ***********/
    /**********************************************/

	/**
	 * @param $method
	 * @param $url
	 * @param array $data
	 *
	 * @return array|bool|mixed
	 */
    public function apiv3Call($method, $url, $data = array()) {

		$headers = array(
			"cache-control: no-cache",
			"Apikey: " . $this->getApiKey(),
			"Pluginkey: " . self::PLUGIN_KEY,
			"Content-Type: application/json",
			"Accept: application/json"
		);

		$url = self::API_V3_URL . $url;

		return $this->call($method, $headers, $url, $data);

	}

	/**
	 * @param $method
	 * @param $url
	 * @param array $data
	 *
	 * @return array|bool|mixed
	 */
	public function transactionalCall($method, $url, $data = array()) {

		$headers = array(
			"Content-Type: application/json",
			"Accept: application/json"
		);

		$data = array_merge($data, array(
			'apikey' => $this->getApiKey(),
			'pluginkey' => self::PLUGIN_KEY
		));

		$url = self::API_TRANSACTIONAL_URL . $url;

		return $this->call($method, $headers, $url, $data);

	}


	/**
	 * Gets all sms senders
	 *
	 * @return array
	 */
	public function getSmsSenders ()
	{
		$senders = json_decode($this->transactionalCall('RAWPOST', '/sms/senders'), true);

		if(!empty($senders['errorCode'])){
			return array();
		}

		foreach ($senders as $key => &$sender) {
			if (!empty($sender['deleted'])) {
				array_splice($senders, $key, $key);
			}
		}

		return $senders;
	}

	/**
	 * Sends transactional sms
	 *
	 * @param $mobile
	 * @param $senderHash
	 * @param $message
	 *
	 * @return array
	 */
	public function sendSms ($mobile, $senderHash, $message)
	{
		$data = array(
			'mobile'     => $mobile,
			'senderHash' => $senderHash,
			'message'    => $message,
			'options'    => array(
				'gsm0338'  => false,
				'maxCount' => self::MAX_MESSAGES
			)
		);

		return $this->transactionalCall('RAWPOST', '/sms/send', $data);
	}


	/**
	 *
	 * Validates a cellphone
	 *
	 * @param null $cellphone
	 * @param null $order
	 *
	 * @return bool|string
	 */
	public function validateCellphone ($cellphone = null, $order = null)
	{

		if (empty($cellphone) || empty($order)) {
			return false;
		}

		$cellphone = str_replace('+', '', $cellphone);

		if (strpos($cellphone, '-') !== false) {
			$cellphone = str_replace(' ', '', $cellphone);
		} else {
			$countryCodes = $this->getEgoiCountryCodes();

			if (!empty($order['payment_iso_code_2'])) {
				$countryInfo = $this->arraySearch($order['payment_iso_code_2'], 'iso_code', $countryCodes);

			} elseif (!empty($order['shipping_iso_code_2'])) {
				$countryInfo = $this->arraySearch($order['payment_iso_code_2'], 'iso_code', $countryCodes);
			}

			if (empty($countryInfo)) {
				return false;
			}

			if (strpos($cellphone, $countryInfo['country_code']) === 0) {
				$cellphone = str_replace('-', ' ', $cellphone);
				$cellphone = str_replace($countryInfo['country_code'], $countryInfo['country_code'] . '-', $cellphone);
				$cellphone = str_replace(' ', '', $cellphone);
			} else {
				$cellphone = $countryInfo['country_code'] . '-' . str_replace(' ', '', $cellphone);
			}
		}

		if (empty($cellphone)) {
			return false;
		}

		$isValidCellphone = $this->transactionalCall('GET', '/sms/validatePhone/' . $cellphone);

		if (!empty($isValidCellphone)) {
			$isValidCellphone = json_decode($isValidCellphone, true);
			if (!empty($isValidCellphone['errorCode'])) {
				return false;
			}
		}

		return $cellphone;
	}

	/**
	 * @return array
	 */
	public function getPluginStatusData()
	{

		$readableVersion = str_split($this->version);
		$readableAvailableVersion = str_split($this->version);

		$values = array(
			'version' => $this->version,
			'readable_version' => implode('.', $readableVersion),
			'available_version' => $this->version,
			'readable_available_version' => implode('.', $readableAvailableVersion),
			'required_update' => false,
			'update_url' => 'https://goidini.e-goi.com/resources/plugins/opencart/smart-marketing-opencart3.latest.ocmod.zip',
			'has_curl' => $this->serverHasCurl(),
			'has_soap' => $this->serverHasSoap(),
			'apiv3' => false,
		);

		$string = file_get_contents(self::PLUGIN_CURRENT_VERSION_URL);

		if($string === FALSE) {
			echo "Could not read the file.";
		} else {
			$pluginsInfo = json_decode($string, true);

			$availableVersion = $pluginsInfo['opencart'];

			if($availableVersion > $this->version){
				$readableAvailableVersion = str_split($availableVersion);
				$values['available_version'] = (int) $availableVersion;
				$values['readable_available_version'] = implode('.', $readableAvailableVersion);
				$values['required_update'] = true;
			}

		}

		$data = $this->apiv3Call('RAWPOST', '/ping');

		if(!empty($data)){
			$data = json_decode($data, true);
			if(!empty($data['result']) && $data['result'] == 'pong'){
				$values['apiv3'] = true;
			}
		}

		return $values;

	}

	/**
	 *
	 * Get a list of country Codes
	 *
	 * @param string $phoneNumber
	 *
	 * @return array
	 */
	protected function getEgoiCountryCodes ($phoneNumber = '')
	{

		$data = array();

		if (!empty($phoneNumber)) {
			$data = ['phone' => $phoneNumber];
		}

		$cc = $this->apiv3Call('GET', '/utilities/countries');

		if (!empty($cc)) {
			$cc = json_decode($cc, true);
			if (!empty($cc['items'])) {
				return $cc['items'];
			}
		}

		return array();

	}

	/**
	 *
	 * search for a key in an array
	 *
	 * @param $value
	 * @param $key
	 * @param $array
	 *
	 * @return int|string|null
	 */
	private function arraySearch($value, $key, $array) {
		foreach ($array as $k => $v) {
			if ($v[$key] === $value) {
				return $v;
			}
		}
		return false;
	}

	/**
	 *
	 * API V3 / Transactional curl wrapper
	 *
	 * @param string $method
	 * @param array $headers
	 * @param string $url
	 * @param array $data
	 *
	 * @return array|bool|mixed
	 */
	private function call($method, $headers = array('Content-Type: application/json'), $url, $data = array()) {

		if(empty($method) || empty($url)) {
			return false;
		}

		$curl = curl_init();

		switch ($method) {
			case "GET":
				if (!empty($data)) {
					$url = sprintf("%s?%s", $url, http_build_query($data));
				}
				break;
			case "POST":
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
				if (!empty($data)) {
					curl_setopt($curl, CURLOPT_POSTFIELDS,http_build_query($data));
				}
				break;
			case "PUT":
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'PUT');
				if ($data) {
					curl_setopt($curl, CURLOPT_POSTFIELDS,http_build_query($data));
				} else {
					return false;
				}
				break;
			case "DELETE":
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'DELETE');
				if (!empty($data)) {
					$url .= (strpos($url, '?') === false ? '?' : '&') . http_build_query($data);
				}
				break;
			case "RAWDELETE":
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'DELETE');
				if ($data) {
					curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
				} else {
					return false;
				}
				break;
			case "OPTIONS":
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'OPTIONS');
				curl_setopt($curl, CURLOPT_HEADER, 1);
				break;
			case "RAWPOST":
				curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
				if (!empty($data)) {
					curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
				}
				break;
			default:
				if (!empty($data)) {
					$url = sprintf("%s?%s", $url, http_build_query($data));
				}
		}

		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_ENCODING, "");
		curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
		curl_setopt($curl, CURLOPT_TIMEOUT, 10);
		curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
		curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);

		$response      = curl_exec($curl);

		curl_close($curl);
		return $response;
	}

	/**
	 * @return bool
	 */
	private function serverHasCurl ()
	{
		return function_exists('curl_init');
	}

	/**
	 * @return bool
	 */
	private function serverHasSoap ()
	{
		return function_exists('curl_init');
	}


}